/*
Odigo Flooder Example
Copyright (C) 2003 Alexander Turkovsky aKa gandalf_g@mail.ru

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//---------------------------------------------------------------------------------------
#include "stdafx.h"
#include "zlib.h"
#include "zlib_compressor.h"


//---------------------------------------------------------------------------------------
ZLib_Compressor::ZLib_Compressor()
{
   LastError = ZLIB_ERROR_OK;
}


//---------------------------------------------------------------------------------------
int ZLib_Compressor::Inflate_MemoryToFile( BYTE *pb, int cb, char *pszFullFn )
{
   HANDLE      h;
   z_stream    d_stream;
   int         err;
   int         cbT;
   int         cbWritten;
   BYTE       *pbInf;
   int         cbInf;

   // Create the dest file
   h = CreateFile( pszFullFn, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 
                   FILE_ATTRIBUTE_NORMAL, NULL );
   if( h == INVALID_HANDLE_VALUE )
   {
      LastError = ZLIB_ERROR_CREATEFILE;
      return 0;
   }

   d_stream.zalloc = (alloc_func)0;
   d_stream.zfree = (free_func)0;
   d_stream.opaque = (voidpf)0;
   d_stream.next_in  = pb;
   d_stream.avail_in = (uInt)cb;
   inflateInit( &d_stream );

   cbInf = 32 * 1024;
   pbInf = (BYTE *)LocalAlloc( LPTR, cbInf );
   if( pbInf == NULL )
   {
      LastError = ZLIB_ERROR_MEMORYALLOCATE;
      goto ZLCIMTF_Error;
   }

   while( 1 )
   {
      // Get next piece

      d_stream.next_out = pbInf;
      d_stream.avail_out = (uInt)cbInf;
      err = inflate( &d_stream, Z_NO_FLUSH );
      if( err < 0 ) goto ZLCIMTF_Error;
      // Write this piece to file
      cbT = cbInf - d_stream.avail_out;
      if( !WriteFile( h, pbInf, cbT, (DWORD*)&cbWritten, NULL ) || cbT != cbWritten )
      {
         LastError = ZLIB_ERROR_WRITEFILE;
         goto ZLCIMTF_Error;
      }

      if( err == Z_STREAM_END ) break;
   }

   LocalFree( (HLOCAL)pbInf );	
   inflateEnd( &d_stream );
   CloseHandle( h );
   Total_In = d_stream.total_in;
   Total_Out = d_stream.total_out;
   return 1;

ZLCIMTF_Error:
   inflateEnd( &d_stream );
   if( pbInf != NULL ) LocalFree( (HLOCAL)pbInf );
   CloseHandle( h );
   DeleteFile( pszFullFn );
   return 0;
}


//---------------------------------------------------------------------------------------
int ZLib_Compressor::Inflate_MemoryToMemory( UCHAR *In, int inlen, UCHAR *Out, int outmaxlen )
{
   z_stream    d_stream;
   int         err;

   d_stream.zalloc = (alloc_func)0;
   d_stream.zfree = (free_func)0;
   d_stream.opaque = (voidpf)0;
   d_stream.next_in  = In;
   d_stream.avail_in = (uInt)inlen;
   d_stream.next_out = Out;
   d_stream.avail_out = (uInt)outmaxlen;

   inflateInit( &d_stream );
   err = inflate( &d_stream, Z_FINISH );
   inflateEnd( &d_stream );

   if( err != Z_STREAM_END && err != Z_OK )
   {
      LastError = ZLIB_ERROR_INFLATE;
      return 0;
   }
   Total_In = d_stream.total_in;
   Total_Out = d_stream.total_out;
   return 1;
}


//---------------------------------------------------------------------------------------
int ZLib_Compressor::Deflate_FileToFile( char *pszSrcFn, char *pszDstFn )
{
   // Open the source file
   char szSrcFn[ MAX_PATH ];
   char *pszFilePart;
   GetFullPathName( pszSrcFn, sizeof( szSrcFn ), szSrcFn, &pszFilePart );

   HANDLE hSrc = CreateFile( szSrcFn, GENERIC_READ, 0, NULL, OPEN_EXISTING, 
                             FILE_ATTRIBUTE_NORMAL, NULL);
   if( hSrc == INVALID_HANDLE_VALUE )
   {
      LastError = ZLIB_ERROR_OPENFILE;
      return 0;
   }

   // Create the dest file

   char szDstFn[ MAX_PATH ];
   GetFullPathName( pszDstFn, sizeof( szDstFn ), szDstFn, &pszFilePart );

   HANDLE hDst = CreateFile( szDstFn, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 
                             FILE_ATTRIBUTE_NORMAL, NULL );
   if( hDst == INVALID_HANDLE_VALUE )
   {
      LastError = ZLIB_ERROR_CREATEFILE;
      return 0;
   }

   z_stream c_stream; /* compression stream */
   int err;

   c_stream.zalloc = (alloc_func)0;
   c_stream.zfree = (free_func)0;
   c_stream.opaque = (voidpf)0;

   err = deflateInit( &c_stream, Z_BEST_COMPRESSION );

   c_stream.next_in = NULL;
   c_stream.avail_in = 0;
   c_stream.next_out = gabBuffOut;
   c_stream.avail_out = (uInt)kcbBuffOut;

   err = Z_OK;
   while( err == Z_OK )
   {
      // Read next uncompressed piece in

      if( c_stream.avail_in == 0 )
      {
         int cbIn;
         ReadFile( hSrc, gabBuffIn, kcbBuffIn, (DWORD *)&cbIn, NULL );
         c_stream.next_in = gabBuffIn;
         c_stream.avail_in = (uInt)cbIn;
      }

      // Compress
      err = deflate( &c_stream, Z_NO_FLUSH );

      // Write compressed bits out
      int cbOut;
      WriteFile( hDst, gabBuffOut, kcbBuffOut - c_stream.avail_out, (DWORD *)&cbOut, NULL );
      c_stream.next_out = gabBuffOut;
      c_stream.avail_out = (uInt)kcbBuffOut;
    }

   err = Z_OK;
   while( err == Z_OK )
   {
      err = deflate( &c_stream, Z_FINISH );

      // Write compressed bits out
      int cbOut;
      WriteFile( hDst, gabBuffOut, kcbBuffOut - c_stream.avail_out, (DWORD *)&cbOut, NULL );
      c_stream.next_out = gabBuffOut;
      c_stream.avail_out = (uInt)kcbBuffOut;
   }
    if( err != Z_STREAM_END && err != Z_OK )
    {
       //printf("Error deflating...\n");
       LastError = ZLIB_ERROR_DEFLATE;
       return 0;
    }
    err = deflateEnd( &c_stream );

   CloseHandle( hSrc );
   CloseHandle( hDst );
   Total_In = c_stream.total_in;
   Total_Out = c_stream.total_out;
   return 1;
}


//---------------------------------------------------------------------------------------
int ZLib_Compressor::Deflate_MemoryToMemory( UCHAR *In, int inlen, UCHAR *Out, int outmaxlen, int CompressionLevel )
{
   z_stream c_stream; /* compression stream */
   int err;

   c_stream.zalloc = (alloc_func)0;
   c_stream.zfree = (free_func)0;
   c_stream.opaque = (voidpf)0;

   err = deflateInit( &c_stream, Z_BEST_COMPRESSION );

   c_stream.next_in = In;
   c_stream.avail_in = inlen;
   c_stream.next_out = Out;
   c_stream.avail_out = (uInt)outmaxlen;

   // Compress
   err = deflate( &c_stream, Z_FINISH );

   if( err != Z_STREAM_END && err != Z_OK )
   {
      LastError = ZLIB_ERROR_DEFLATE;
      return 0;
   }
   deflateEnd( &c_stream );
   Total_In = c_stream.total_in;
   Total_Out = c_stream.total_out;
   return 1;
}

